%%%----------------------------------------------------------------------
%%% File    : demo.erl
%%% Author  : Vlad Dumitrescu <qrvvldu@sixten9>
%%% Purpose : 
%%% Created : 11 Mar 2003 by Vlad Dumitrescu <qrvvldu@sixten9>
%%%----------------------------------------------------------------------

-module(demo).
-author('qrvvldu@sixten9').

-compile(export_all).

-behaviour(gen_server).

%% External exports
-export([start_link/0]).

%% gen_server callbacks
-export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2, code_change/3]).

-record(state, {dummy=true}).

%%%----------------------------------------------------------------------
%%% API
%%%----------------------------------------------------------------------
start_link() ->
    gen_server_2:start_link({local, demo}, [demo, demo, demo, demo, demo], [], []).

stop() ->
    gen_server_2:call(demo, stop).

test(Arg) ->
    gen_server_2:call(demo, {test, Arg}).

hello() ->
    gen_server_2:call(demo, hello).

test2(Arg) ->
    gen_server_2:cast(demo, {test2, Arg}).

%%%----------------------------------------------------------------------
%%% Callback functions from gen_server
%%%----------------------------------------------------------------------

%%----------------------------------------------------------------------
%% Func: init/1
%% Returns: {ok, State}          |
%%          {ok, State, Timeout} |
%%          ignore               |
%%          {stop, Reason}
%%----------------------------------------------------------------------
init({Args, Mods}) ->
    {ok, {#state{}, Mods}}.

%%----------------------------------------------------------------------
%% Func: handle_call/3
%% Returns: {reply, Reply, State}          |
%%          {reply, Reply, State, Timeout} |
%%          {noreply, State}               |
%%          {noreply, State, Timeout}      |
%%          {stop, Reason, Reply, State}   | (terminate/2 is called)
%%          {stop, Reason, State}            (terminate/2 is called)
%%----------------------------------------------------------------------
handle_call(stop, _From, State) ->
    {stop, normal, [], State};
handle_call(hello, From, {S, _Mods}=State) ->
    Reply = S#state.dummy,
    {reply, Reply, State};
handle_call(Request, From, {_S, Mods}=State) ->
    io:format("call: ~w // ~w~n", [Request, State]),
    case gen_server_2:next_call(Request, From, State) of
	{reply, R, {St, _M}=St2} ->
	    io:format("   R: ~w ~w~n", [R, St2]),
	    Reply = St#state.dummy,
	    {reply, Reply, {St#state{dummy=not Reply}, Mods}};
	error ->
	    {reply, true, State}
    end.

%%----------------------------------------------------------------------
%% Func: handle_cast/2
%% Returns: {noreply, State}          |
%%          {noreply, State, Timeout} |
%%          {stop, Reason, State}            (terminate/2 is called)
%%----------------------------------------------------------------------
handle_cast(Msg, State) ->
    io:format("cast: ~w // ~w~n", [Msg, State]),
    gen_server_2:next_cast(Msg, State),
    {noreply, State}.

%%----------------------------------------------------------------------
%% Func: handle_info/2
%% Returns: {noreply, State}          |
%%          {noreply, State, Timeout} |
%%          {stop, Reason, State}            (terminate/2 is called)
%%----------------------------------------------------------------------
handle_info(Info, State) ->
    {noreply, State}.

%%----------------------------------------------------------------------
%% Func: terminate/2
%% Purpose: Shutdown the server
%% Returns: any (ignored by gen_server)
%%----------------------------------------------------------------------
terminate(Reason, State) ->
    ok.

%%----------------------------------------------------------------------
%% Func: code_change/3
%% Purpose: Convert process state when code is changed
%% Returns: {ok, NewState}
%%----------------------------------------------------------------------
code_change(OldVsn, State, Extra) ->
    {ok, State}.

%%%----------------------------------------------------------------------
%%% Internal functions
%%%----------------------------------------------------------------------
